@@ -66,12 +66,12 @@ impl fmt::Display for TypedResponseError {
         write!(f, "Error converting response: ")?;
 
         match &self.kind {
-            ErrorKind::Missing => write!(f, "field {:?} is but missing", self.field),
+            ErrorKind::Missing => write!(f, "field {:?} is required but missing", self.field),
             ErrorKind::InvalidValue(val) => {
                 write!(f, "value {:?} is invalid for field {:?}", val, self.field)
             }
             ErrorKind::UnexpectedField(found) => {
-                write!(f, "Expected field {:?} but found {:?}", self.field, found)
+                write!(f, "expected field {:?} but found {:?}", self.field, found)
             }
             ErrorKind::MalformedInteger(_) => write!(f, "field {:?} is not an integer", self.field),
             ErrorKind::MalformedFloat(_) => write!(f, "field {:?} is not a float", self.field),
@@ -121,6 +121,7 @@ pub enum PlayState {
 /// [status-command]: https://www.musicpd.org/doc/html/protocol.html#command-status
 #[derive(Clone, Debug, PartialEq, Eq)]
 #[allow(missing_docs)]
+#[non_exhaustive]
 pub struct Status {
     pub volume: u8,
     pub state: PlayState,
@@ -161,6 +162,23 @@ impl Response for Status {
             },
         };
 
+        let duration = if let Some(val) = raw.get("duration") {
+            Some(parse!(duration, val, "duration"))
+        } else if let Some(time) = raw.get("time") {
+            // Backwards compatibility with protocol versions <0.20
+            if let Some((_, duration)) = time.split_once(':') {
+                Some(parse!(duration, duration, "time"))
+            } else {
+                // No separator
+                return Err(TypedResponseError {
+                    field: "time",
+                    kind: ErrorKind::InvalidValue(time),
+                });
+            }
+        } else {
+            None
+        };
+
         let mut partition = raw.get("partition");
 
         if partition.as_deref() == Some("default") {
@@ -179,7 +197,7 @@ impl Response for Status {
             current_song: song_identifier!(raw, "song", "songid"),
             next_song: song_identifier!(raw, "nextsong", "nextsongid"),
             elapsed: field!(raw, "elapsed" duration optional),
-            duration: field!(raw, "duration" duration optional),
+            duration,
             bitrate: field!(raw, "bitrate" integer optional),
             crossfade: field!(raw, "xfade" duration default Duration::from_secs(0)),
             update_job: field!(raw, "update_job" integer optional),
@@ -194,6 +212,7 @@ impl Response for Status {
 /// [`stats`]: crate::commands::definitions::Stats
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 #[allow(missing_docs)]
+#[non_exhaustive]
 pub struct Stats {
     pub artists: u64,
     pub albums: u64,
