@@ -2,7 +2,6 @@ use chrono::{DateTime, FixedOffset};
 
 use std::cmp;
 use std::collections::HashMap;
-use std::convert::TryFrom;
 use std::iter;
 use std::num::ParseIntError;
 use std::path::Path;
@@ -16,6 +15,7 @@ use crate::tag::Tag;
 ///
 /// [`playlistinfo`]: crate::commands::definitions::Queue
 #[derive(Clone, Debug, PartialEq, Eq)]
+#[non_exhaustive]
 pub struct SongInQueue {
     /// Position in queue.
     pub position: SongPosition,
@@ -81,6 +81,7 @@ impl SongInQueue {
 /// [playlist]: crate::commands::definitions::Queue
 /// [current song]: crate::commands::definitions::CurrentSong
 #[derive(Clone, Debug, PartialEq, Eq)]
+#[non_exhaustive]
 pub struct Song {
     /// Unique identifier of the song. May be a file path relative to the library root, or an URL
     /// to a remote resource.
@@ -202,23 +203,10 @@ where
     type Item = Result<(Song, Option<SongQueueData>), TypedResponseError>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        let (mut key, mut value) = self.fields.next()?;
-
-        // Skip directory entries, encountered when using e.g. the `listallinfo` command.
-        if key.as_ref() == "directory" {
-            loop {
-                let next = self.fields.next()?;
-                match next.0.as_ref() {
-                    "directory" => continue,
-                    "Last-Modified" => continue,
-                    _ => {
-                        key = next.0;
-                        value = next.1;
-                        break;
-                    }
-                }
-            }
-        }
+        let (key, value) = self
+            .fields
+            // Skip directory entries, encountered when using e.g. the `listallinfo` command.
+            .find(|(k, _)| &**k != "directory" && &**k != "Last-Modified")?;
 
         let mut song = if key.as_ref() == "file" {
             Song::new(value)
@@ -257,7 +245,18 @@ where
                         }))
                     }
                 },
-                // Just a worse "duration" field.
+                // Just a worse "duration" field, but needed for backwards compatibility with
+                // protocol versions <0.20
+                "Time" if song.duration.is_none() => match value.parse() {
+                    Ok(v) => song.duration = Some(Duration::from_secs_f64(v)),
+                    Err(e) => {
+                        return Some(Err(TypedResponseError {
+                            field: "Time",
+                            kind: ErrorKind::MalformedFloat(e),
+                        }))
+                    }
+                },
+                // If we already have a duration value, ignore the Time field
                 "Time" => (),
                 "Range" => {
                     range = match parse_range_field(value) {
@@ -441,6 +440,44 @@ mod tests {
         );
     }
 
+    #[test]
+    fn song_parser_deprecated_time_field() {
+        let input = key_value_pairs(vec![("file", "foo/bar.baz"), ("Time", "123")]);
+
+        let songs = Song::parse_frame(input, None).unwrap();
+
+        assert_eq!(
+            songs,
+            vec![Song {
+                url: String::from("foo/bar.baz"),
+                duration: Some(Duration::from_secs(123)),
+                tags: HashMap::new(),
+                format: None,
+                last_modified: None,
+            }]
+        );
+
+        // Duration field takes priority:
+        let input = key_value_pairs(vec![
+            ("file", "foo/bar.baz"),
+            ("duration", "456.789"),
+            ("Time", "123"),
+        ]);
+
+        let songs = Song::parse_frame(input, None).unwrap();
+
+        assert_eq!(
+            songs,
+            vec![Song {
+                url: String::from("foo/bar.baz"),
+                duration: Some(Duration::from_secs_f64(456.789)),
+                tags: HashMap::new(),
+                format: None,
+                last_modified: None,
+            }]
+        );
+    }
+
     #[test]
     fn song_in_queue_parser() {
         let ts = "2020-06-12T17:53:00Z";
